@@ -51,7 +51,7 @@ class ElwaSc20(SerialDevice):
             self._devicesubtype = "SC20"
             self._deviceBoostpower = 2000
 
-        self._buscommunicationWaitTime = 30
+        self._buscommunicationWaitTime = 5
         self._datamaximumage = 90
 
         self._channels["fw_version"] = ChannelConfig([0], None, 0, registertype="serial")
@@ -79,11 +79,11 @@ class ElwaSc20(SerialDevice):
         self._channels["internal_control_value_3"] = ChannelConfig([22], "NUMBER", 0, registertype="serial")
         self._channels["internal_clock"] = ChannelConfig([23], "NUMBER", 0, registertype="serial")
         self._channels["minutes_since_wakeup"] = ChannelConfig([24], "NUMBER", 0, registertype="serial")
-        self._channels["ac_boost_mode"] = ChannelConfig([25], "NUMBER", 0, registertype="serial")
+        self._channels["ac_boost_mode"] = ChannelConfig([25], "NUMBER", 0, registertype="serial", mode="rw") # ac switch on time. 0: temperature controlled
         self._channels["is_master_device"] = ChannelConfig([26], "NUMBER", 0, registertype="serial")
-        self._channels["serial"] = ChannelConfig([27], None, 0, registertype="serial")
-        self._channels["internal_control_value_4"] = ChannelConfig([28], "NUMBER", 0, registertype="serial")
-        # self._channels["8bit_checksum"] = ChannelConfig([29], "NUMBER", 0, registertype="serial")
+        self._channels["serial"] = ChannelConfig([27], None, 0, registertype="serial", mode="rw")
+        self._channels["pot_position"] = ChannelConfig([28], "NUMBER", 0, registertype="serial")
+        self._channels["useacboost"] = ChannelConfig([30], "NUMBER", 0, registertype="serial", mode="rw")
 
         self._setup["fwversion"] = SetupMapping(defaultvalue=self._firmwareversion) #required to find write configuration in cloud
         self._setup["device"] = SetupMapping(defaultvalue=self._devicesubtype)
@@ -91,6 +91,8 @@ class ElwaSc20(SerialDevice):
         self._setup["libversion"] = SetupMapping(defaultvalue=self._firmwareversion)
         self._setup["serialno"] = SetupMapping(defaultvalue=self._serial)
         self._setup["ww1boost"] = SetupMapping("ac_temp_target")
+        self._setup["booststarttime"] = SetupMapping("ac_boost_mode")     #time when we start to boost in minutes after 
+        self._setup["useacboost"] = SetupMapping("useacboost")
 
     def _checkid(self, portname):
         try:
@@ -107,45 +109,50 @@ class ElwaSc20(SerialDevice):
             return True
         return False
 
-    def _writeregister(self, registerId, valueToWrite):
-        if registerId != 10:
-            raise Exception("Invalid address")
+    def _lookupcommand(self, registerId):
+        if registerId == 10:
+            return "sact"
+        if registerId == 27:
+            return "ssn"
+        elif registerId == 25:
+            return "acmode"
+        else:
+            raise Exception("invalid register id: " + str(registerId))
 
-        if valueToWrite / 10 < ACTEMPMINIMUMVALUE:
-            raise Exception("Value to low: " + str(valueToWrite))
+    def _readserialdata(self):
+        data = super()._readserialdata()
+        serial = list(data[27])
+        acmode = serial[2]
+        serial[2] = "0"
+        serialfixed = "".join(serial)
+        data[27] = serialfixed
+        data.append(acmode)
+        return data
 
-        if valueToWrite / 10 > ACTEMPMAXIMUMVALUE:
-            raise Exception("Value to high: " + str(valueToWrite))
-
-        super()._writeregister(registerId, valueToWrite)
-
-    def _writeregisters(self, registerStartId, valuesToWrite):
-        if registerStartId != 10 and len(registerStartId) != 1:
-            raise Exception("Invalid address")
-        super()._writeregister(registerStartId, int(valuesToWrite[0]))
+    def _writeserialdata(self, registerId, data):
+        if registerId == 30:
+            # if not isinstance(data, bool):
+            if data != 0 and data != 1:
+                raise Exception("invalid value for register 30")
+            registerId = 27
+            serial = list(self._serial)
+            serial[2] = str(data)
+            data = "".join(serial)
+        super()._writeserialdata(registerId, data)
 
     def getdata(self):
         acRelayState = self.getchannelvalue("ac_relay_state", True)
         dcPower = self.getchannelvalue("dc_power", True)
         operationMode = self.getchannelvalue("operation_mode", True)
-        acdaywh = self.getchannelvalue("ac_day_wh", True)
+        useacboost = self.getchannelvalue("useacboost", True)
 
         if operationMode == 3 or operationMode == 5 or operationMode == 6 or operationMode == 7 or operationMode == 8 or operationMode == 9 or operationMode == 10 or operationMode == 11 or operationMode == 12 or operationMode == 13 or operationMode == 14 or operationMode == 15 or operationMode == 16 or operationMode == 20 or operationMode == 21 or operationMode == 135:
             acHeating = True
         else:
             acHeating = False
 
-        if self._oldacdaywh != None:
-            if self._oldacdaywh != acdaywh:
-                acchanged = True
-            else:
-                acchanged = False
-        else:
-            self._oldacdaywh = acdaywh
-            acchanged = False
-
         if acRelayState != None:
-            if acHeating == True and acRelayState == 1 and acchanged == True:
+            if acHeating == True and acRelayState == 1 and useacboost == 1:
                 boostpower = self._deviceBoostpower
             else:
                 boostpower = 0
@@ -153,7 +160,7 @@ class ElwaSc20(SerialDevice):
             boostpower = None
 
         if(boostpower != None):
-            power = (int(dcPower) + int(boostpower))
+            power = dcPower + boostpower
             meter = -boostpower
         else:
             power = dcPower
@@ -173,7 +180,7 @@ class ElwaSc20(SerialDevice):
             "op_mode": operationMode,
             "dc_breaker": self.getchannelvalue("dc_breaker_state", True),
             "dc_relay": self.getchannelvalue("dc_relay_state", True),
-            "ac_relay": acRelayState,
+            "ac_boost": acRelayState,
             "temp1": self.getchannelvalue("water_temp", True, 10),
             "temp_day_min": self.getchannelvalue("temp_day_min", True, 10),
             "temp_day_max": self.getchannelvalue("temp_day_max", True, 10),
@@ -183,11 +190,11 @@ class ElwaSc20(SerialDevice):
             "iso_voltage": self.getchannelvalue("iso_voltage", True),
             "dc_voltage": self.getchannelvalue("dc_voltage", True, 10),
             "dc_current": self.getchannelvalue("dc_current", True, 1000),
-            "boostpower": boostpower,
+            "boostpower_sc20": boostpower,
             "dc_day_wh": self.getchannelvalue("dc_day_wh", True),
             "dc_total_kwh": self.getchannelvalue("dc_total_kwh", True),
-            "ac_day_wh": acdaywh,
-            "ac_boost_mode": self.getchannelvalue("ac_boost_mode", True),
+            "ac_day_wh": self.getchannelvalue("ac_day_wh", True),
+            # "ac_boost_mode": self.getchannelvalue("ac_boost_mode", True),
             "m1sum": dcPower,
             "m0sum": meter,
             "buserrorrate":  errorrate
@@ -203,22 +210,9 @@ class ElwaSc20(SerialDevice):
 
     def getlogdata(self, time = None):
         acRelayState = self.getlogvalue("ac_relay_state")
-        acdaywh = self.getlogvalue("ac_day_wh")
-
-        if self._oldacdaywhlog != None:
-            if self._oldacdaywhlog != acdaywh:
-                acchanged = True
-            else:
-                acchanged = False
-        else:
-            self._oldacdaywhlog = acdaywh
-            acchanged = False
 
         if acRelayState != None:
-            if acchanged == False:
-                boostpower = 0
-            else:
-                boostpower = int(round(acRelayState*self._deviceBoostpower))
+            boostpower = int(round(acRelayState * self._deviceBoostpower))
         else:
             boostpower = None
         
@@ -251,6 +245,21 @@ class ElwaSc20(SerialDevice):
 
         return logData
 
+    def _checksumcheck(self, response_string, receivedchecksum):
+        return
+        # checksum = 0
+        # lasttab = response_string.rfind('\\t')
+        # payloadstring = response_string[2:lasttab]
+        # # payloadstring += "\\r\\n'"
+        # for c in payloadstring:
+        #     value = ord(c)
+        #     print(value)
+        #     checksum += value
+        #     checksum = checksum % 255
+
+        # if receivedchecksum != checksum:
+        #     raise Exception("Checksum missmatch.")
+
     def _supervise(self):
         super()._supervise()
         self._sethealthstate(0) # Todo
@@ -271,7 +280,65 @@ if __name__ == "__main__":
     dcscryptokey = "41424142414241424142414241424142"
     server = "my-pv.live"
 
-    # device = ElwaSc20(serialnr)
+    device = ElwaSc20(serialnr)
+    # device.readallregisters()
+    # print(device)
+    # useacboost = device.getchannelvalue("useacboost")
+    
+    # device.writeregister(30, 1)
+    # device.readallregisters()
+    # print(device)
+    device.setsetupvalue("useacboost", 1)
+    # device.readallregisters()
+    # print(device)
+    # device.setsetupvalue("useacboost", 0)
+    # device.readallregisters()
+    # print(device)
+    # print()
+
+    # device.writeregister(10, 532)
+    # device.writeregister(27, "1203002111190000")
+    # device.writeregister(25, 200)
+    # device.readallregisters()
+    # print(device)
+
+    # device.readallregisters()
+    # print(device)
+    # print(device.readregister(27, registertype='serial'))
+    # print(device.readregisters(27, 2, registertype='serial'))
+    # print(device)
+    # try:
+    #     device.writeregister(10, 532)
+    # except Exception as e:
+    #     print(e)
+    # device.readallregisters()
+    # print(device)
+
+    # device.setsetupvalue("ww1boost", 50)
+    # device.readallregisters()
+    # print(device)
+
+    # try:
+    #     device.writeregister(25, 200)
+    # except Exception as e:
+    #     print(e)
+    # device.readallregisters()
+    # print(device)
+
+    # try:
+    #     device.writeregister(27, "1203002111190000")
+    # except Exception as e:
+    #     print(e)
+    # device.readallregisters()
+    # print(device)
+
+    # print()
+
+    # device.setsetupvalue("booststarttime", 205)
+    # device.readallregisters()
+    # print(device)
+    # print()
+
     # device2 = ElwaSc20("1201061912110002")
     # device.readallregisters()
     # device2.readallregisters()
