@@ -112,9 +112,17 @@ class SerialDevice(GeneralDevice):
             logging.warning("Cannot read from device " + str(self._name) + ". " + str(e))
             raise e
 
+        key = "b'dr" # Todo Abhängig vom Gerätetyp ...
         if len(response) > 0:
-            response_string = str(response[0])
-            content_string = response_string[4:-5]      # Todo Abhängig vom Gerätetyp zuschneiden...
+            response_string = None
+            for element in response:
+                if str(element).startswith(key):
+                    response_string = str(element)
+                    break
+            if response_string == None:
+                raise Exception("no valid response received: " + str(response))
+            
+            content_string = response_string[len(key):-(len(key)+1)]      
         else:
             # raise Exception("Noting received from device " + str(self._name) + ".")
             logging.debug("Noting received from device " + str(self._name) + ".")
@@ -127,8 +135,17 @@ class SerialDevice(GeneralDevice):
         except Exception as e:
             raise Exception("Cannot split content_string. " + str(e))
 
+        try:
+            receivedchecksum = int(received_values[29])       # todo device specific checksume calculation
+            self._checksumcheck(response_string, receivedchecksum)
+        except Exception as e:
+            raise Exception("Checksum check failed: " + str(e)) 
+
         return received_values
 
+    def _checksumcheck(self, response_string, receivedchecksum):
+        return
+
     def _readregister(self, registerId, registertype="serial"):
         if registertype != "serial":
             raise Exception("Register type not supportet. Only serial is supported by this device type. Register: " + str(registertype))  
@@ -145,7 +162,8 @@ class SerialDevice(GeneralDevice):
             raise Exception("Serial communication failed.")
         except Exception as e:
             self._increaseBusErrorCounter()
-            self._commport.close()
+            if self._commport != None:
+                self._commport.close()
             raise Exception("Register " + str(registertype) + ":" + str(registerId) + " cannot be read. " + str(e))
 
     def _readregisters(self, startregisteraddress, registerstoread, registertype="serial"):
@@ -167,38 +185,49 @@ class SerialDevice(GeneralDevice):
             raise Exception("Serial communication failed.")
         except Exception as e:
             self._increaseBusErrorCounter()
-            self._commport.close()
+            if self._commport != None:
+                self._commport.close()
             raise Exception("Registers " + str(registertype) + ":" + str(startregisteraddress) + ":" + str(registerstoread) + " cannot be read. " + str(e))
 
-    def _writeserialdata(self, data):
+    def _writeserialdata(self, registerId, data):
         if self._commport == None:
             logging.warning("No connection established.")
             raise Exception("not connected")
 
+        command = self._lookupcommand(registerId)
+        request = (command + ' {}\r\n'.format(data)).encode('utf-8')
         self._busconnectioncounter += 1
         try:
-            self._commport.write('sact {}\r\n'.format(data).encode('utf-8'))
+            self._commport.write(request)
+            self._commport.readlines()
         except Exception as e:
             logging.warning("Cannot write data to device " + str(self._name) + ". Data: " + str(data) + ". Error: " + str(e))
             raise e
+    
+    def _lookupcommand(self, registerId):
+        raise Exception("not implemented")
 
     def _writeregister(self, registerId, valueToWrite):
         logging.debug(str(self._name) + " writing register " + str(registerId) + " value: " + str(valueToWrite))
 
-        self._writeserialdata(valueToWrite)
+        self._writeserialdata(registerId, valueToWrite)
         try:
             valuefromdevice = self._readregister(registerId)
         except Exception as e:
             logging.debug(str(self._name) + " cannot verify written data.")
             self._increaseBusErrorCounter()
-            raise Exception("wraiting failed. Cannot verify")
+            raise Exception("writing failed. Cannot verify")
         try:
-            valuefromdevice = int(valuefromdevice)
+            valuefromdevice = str(valuefromdevice)
+        except Exception as e:
+            pass
+        try:
+            valueToWrite = str(valueToWrite)
         except Exception as e:
             pass
         if valueToWrite != valuefromdevice:
             self._increaseBusErrorCounter()
-            raise Exception("wraiting failed. Value missmatch: " + str(valuefromdevice))
+            raise Exception("writing failed. Value missmatch. Device: " + str(valuefromdevice) + "; Setup: " + str(valueToWrite))
 
     # def _writeregisters(self, registerStartId, valuesToWrite):        # todo
     #     logging.debug("[ModbusRtuDevice] ID " + str(self._serial) + " writing registers " + str(registerIds) + " value: " + str(valuesToWrite))
